from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class AssetsGetResponse_pagination(Parsable):
    # An opaque cursor token that identifies where the next page of paginated results should start. It'sreturned in each paginated response so that it can be supplied in the next request to continue paginatedresults. If a paginated response contains no ``cursorState`` value, then there are no further pages toreturn.Omit this field to initiate a paginated request or to restart pagination.
    cursor_state: Optional[str] = None
    # The maximum number of objects that can be returned in a page. A request might return fewer objects thanthe limit if the Assets service runs out of specified objects to return - at the end of a set of pagedresults, for example. The maximum limit is ``200``; the default limit is ``25``.
    limit: Optional[int] = None
    # A URL that requests the next page for this query.
    next_url: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> AssetsGetResponse_pagination:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: AssetsGetResponse_pagination
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return AssetsGetResponse_pagination()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "cursorState": lambda n : setattr(self, 'cursor_state', n.get_str_value()),
            "limit": lambda n : setattr(self, 'limit', n.get_int_value()),
            "nextUrl": lambda n : setattr(self, 'next_url', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("cursorState", self.cursor_state)
        writer.write_int_value("limit", self.limit)
        writer.write_str_value("nextUrl", self.next_url)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .users_import_post_request_body_users_products_access import UsersImportPostRequestBody_users_products_access

@dataclass
class UsersImportPostRequestBody_users_products(Parsable):
    # The user's type of access to the product identified by ``key``. Possible values:- ``administrator``- ``member``- ``none``Note that when you're using a POST or PATCH endpoint to set this value, you must adhere to the following guidelines:- If you set a product's ``key`` to ``projectAdministration`` and you set ``access`` to ``none``, all other products should be set to ``member`` access for the user.- If you set a product's ``key`` to ``projectAdministration`` and you set ``access`` to ``administrator``, all other products should be set to ``administrator`` access for the user.- You cannot set a product's ``key`` to ``projectAdministration`` and set ``access`` to ``member``.
    access: Optional[UsersImportPostRequestBody_users_products_access] = None
    # A keyword that identifies the product.Possible values: ``autoSpecs``, ``build``, ``cost``, ``designCollaboration``, ``docs``, ``insight``, ``modelCoordination``, ``projectAdministration``, and ``takeoff``.
    key: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> UsersImportPostRequestBody_users_products:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: UsersImportPostRequestBody_users_products
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return UsersImportPostRequestBody_users_products()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .users_import_post_request_body_users_products_access import UsersImportPostRequestBody_users_products_access

        from .users_import_post_request_body_users_products_access import UsersImportPostRequestBody_users_products_access

        fields: dict[str, Callable[[Any], None]] = {
            "access": lambda n : setattr(self, 'access', n.get_enum_value(UsersImportPostRequestBody_users_products_access)),
            "key": lambda n : setattr(self, 'key', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_enum_value("access", self.access)
        writer.write_str_value("key", self.key)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class WithErrorCodeNameGetResponse(Parsable):
    # A description of the error.
    description: Optional[str] = None
    # The HTTP status code typically returned when this error occurs.Note that there may be exceptions, for example, if an internal server erroroccurs the status code associated with this error may be different.
    http_status_code: Optional[float] = None
    # The metadata keys relevant to the error code.
    metadata_keys: Optional[list[str]] = None
    # The error code name.
    name: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithErrorCodeNameGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithErrorCodeNameGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithErrorCodeNameGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "httpStatusCode": lambda n : setattr(self, 'http_status_code', n.get_float_value()),
            "metadataKeys": lambda n : setattr(self, 'metadata_keys', n.get_collection_of_primitive_values(str)),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("description", self.description)
        writer.write_float_value("httpStatusCode", self.http_status_code)
        writer.write_collection_of_primitive_values("metadataKeys", self.metadata_keys)
        writer.write_str_value("name", self.name)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .classification_systems_post_request_body_classifications_measurement_type import ClassificationSystemsPostRequestBody_classifications_measurementType

@dataclass
class ClassificationSystemsPostRequestBody_classifications(Parsable):
    # The classification code.Max length: 256
    code: Optional[str] = None
    # A description of the classification.Max length: 256
    description: Optional[str] = None
    # **Deprecated. Will be removed on September 15, 2025.**The type of measurement.Possible values: ``AREA``, ``COUNT``, ``DISTANCE``, ``VOLUME``.
    measurement_type: Optional[ClassificationSystemsPostRequestBody_classifications_measurementType] = None
    # The classification parent code.Its value may be ``null``, indicating that this classification is at the top level of the hierarchy.Max length: 256
    parent_code: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ClassificationSystemsPostRequestBody_classifications:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ClassificationSystemsPostRequestBody_classifications
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ClassificationSystemsPostRequestBody_classifications()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .classification_systems_post_request_body_classifications_measurement_type import ClassificationSystemsPostRequestBody_classifications_measurementType

        from .classification_systems_post_request_body_classifications_measurement_type import ClassificationSystemsPostRequestBody_classifications_measurementType

        fields: dict[str, Callable[[Any], None]] = {
            "code": lambda n : setattr(self, 'code', n.get_str_value()),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "measurementType": lambda n : setattr(self, 'measurement_type', n.get_enum_value(ClassificationSystemsPostRequestBody_classifications_measurementType)),
            "parentCode": lambda n : setattr(self, 'parent_code', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("code", self.code)
        writer.write_str_value("description", self.description)
        writer.write_enum_value("measurementType", self.measurement_type)
        writer.write_str_value("parentCode", self.parent_code)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .with_response_patch_response_rfi_permitted_actions_update_rfi_permitted_attributes import WithResponsePatchResponse_rfi_permittedActions_updateRfi_permittedAttributes
    from .with_response_patch_response_rfi_permitted_actions_update_rfi_permitted_statuses import WithResponsePatchResponse_rfi_permittedActions_updateRfi_permittedStatuses

@dataclass
class WithResponsePatchResponse_rfi_permittedActions_updateRfi(Parsable):
    """
    Information about the user who most recently updated the RFI.
    """
    # The list of attributes that are required when creating an RFI.
    permitted_attributes: Optional[list[WithResponsePatchResponse_rfi_permittedActions_updateRfi_permittedAttributes]] = None
    # A list of statuses the user is permitted to transition an RFI to.
    permitted_statuses: Optional[WithResponsePatchResponse_rfi_permittedActions_updateRfi_permittedStatuses] = None
    # ``true``: The user is allowed to fill in custom attributes when creating the RFI.``false`` (default): The user is not allowed to fill in custom attributes when creating the RFI.
    use_custom_attributes: Optional[bool] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithResponsePatchResponse_rfi_permittedActions_updateRfi:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithResponsePatchResponse_rfi_permittedActions_updateRfi
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithResponsePatchResponse_rfi_permittedActions_updateRfi()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .with_response_patch_response_rfi_permitted_actions_update_rfi_permitted_attributes import WithResponsePatchResponse_rfi_permittedActions_updateRfi_permittedAttributes
        from .with_response_patch_response_rfi_permitted_actions_update_rfi_permitted_statuses import WithResponsePatchResponse_rfi_permittedActions_updateRfi_permittedStatuses

        from .with_response_patch_response_rfi_permitted_actions_update_rfi_permitted_attributes import WithResponsePatchResponse_rfi_permittedActions_updateRfi_permittedAttributes
        from .with_response_patch_response_rfi_permitted_actions_update_rfi_permitted_statuses import WithResponsePatchResponse_rfi_permittedActions_updateRfi_permittedStatuses

        fields: dict[str, Callable[[Any], None]] = {
            "permittedAttributes": lambda n : setattr(self, 'permitted_attributes', n.get_collection_of_object_values(WithResponsePatchResponse_rfi_permittedActions_updateRfi_permittedAttributes)),
            "permittedStatuses": lambda n : setattr(self, 'permitted_statuses', n.get_object_value(WithResponsePatchResponse_rfi_permittedActions_updateRfi_permittedStatuses)),
            "useCustomAttributes": lambda n : setattr(self, 'use_custom_attributes', n.get_bool_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_object_values("permittedAttributes", self.permitted_attributes)
        writer.write_object_value("permittedStatuses", self.permitted_statuses)
        writer.write_bool_value("useCustomAttributes", self.use_custom_attributes)
    




from enum import Enum

class ContentViewsGetResponse_results_view_SheetView_calibration_units(str, Enum):
    FT_AND_DECIMAL_IN = "FT_AND_DECIMAL_IN",
    FT_AND_FRACTIONAL_IN = "FT_AND_FRACTIONAL_IN",
    M = "M",
    CM = "CM",
    MM = "MM",




from __future__ import annotations
from collections.abc import Callable
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .collections.collections_request_builder import CollectionsRequestBuilder
    from .exports.exports_request_builder import ExportsRequestBuilder
    from .sheets.sheets_request_builder import SheetsRequestBuilder
    from .sheets_batch_delete.sheets_batch_delete_request_builder import SheetsBatchDeleteRequestBuilder
    from .sheets_batch_get.sheets_batch_get_request_builder import SheetsBatchGetRequestBuilder
    from .sheets_batch_restore.sheets_batch_restore_request_builder import SheetsBatchRestoreRequestBuilder
    from .sheets_batch_update.sheets_batch_update_request_builder import SheetsBatchUpdateRequestBuilder
    from .storage.storage_request_builder import StorageRequestBuilder
    from .uploads.uploads_request_builder import UploadsRequestBuilder
    from .version_sets.version_sets_request_builder import VersionSetsRequestBuilder
    from .version_sets_batch_delete.version_sets_batch_delete_request_builder import VersionSetsBatchDeleteRequestBuilder
    from .version_sets_batch_get.version_sets_batch_get_request_builder import VersionSetsBatchGetRequestBuilder

class WithProjectItemRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/sheets/v1/projects/{projectId}
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new WithProjectItemRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/sheets/v1/projects/{projectId}", path_parameters)
    
    @property
    def collections(self) -> CollectionsRequestBuilder:
        """
        The collections property
        """
        from .collections.collections_request_builder import CollectionsRequestBuilder

        return CollectionsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def exports(self) -> ExportsRequestBuilder:
        """
        The exports property
        """
        from .exports.exports_request_builder import ExportsRequestBuilder

        return ExportsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def sheets(self) -> SheetsRequestBuilder:
        """
        The sheets property
        """
        from .sheets.sheets_request_builder import SheetsRequestBuilder

        return SheetsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def sheets_batch_delete(self) -> SheetsBatchDeleteRequestBuilder:
        """
        The sheetsBatchDelete property
        """
        from .sheets_batch_delete.sheets_batch_delete_request_builder import SheetsBatchDeleteRequestBuilder

        return SheetsBatchDeleteRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def sheets_batch_get(self) -> SheetsBatchGetRequestBuilder:
        """
        The sheetsBatchGet property
        """
        from .sheets_batch_get.sheets_batch_get_request_builder import SheetsBatchGetRequestBuilder

        return SheetsBatchGetRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def sheets_batch_restore(self) -> SheetsBatchRestoreRequestBuilder:
        """
        The sheetsBatchRestore property
        """
        from .sheets_batch_restore.sheets_batch_restore_request_builder import SheetsBatchRestoreRequestBuilder

        return SheetsBatchRestoreRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def sheets_batch_update(self) -> SheetsBatchUpdateRequestBuilder:
        """
        The sheetsBatchUpdate property
        """
        from .sheets_batch_update.sheets_batch_update_request_builder import SheetsBatchUpdateRequestBuilder

        return SheetsBatchUpdateRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def storage(self) -> StorageRequestBuilder:
        """
        The storage property
        """
        from .storage.storage_request_builder import StorageRequestBuilder

        return StorageRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def uploads(self) -> UploadsRequestBuilder:
        """
        The uploads property
        """
        from .uploads.uploads_request_builder import UploadsRequestBuilder

        return UploadsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def version_sets(self) -> VersionSetsRequestBuilder:
        """
        The versionSets property
        """
        from .version_sets.version_sets_request_builder import VersionSetsRequestBuilder

        return VersionSetsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def version_sets_batch_delete(self) -> VersionSetsBatchDeleteRequestBuilder:
        """
        The versionSetsBatchDelete property
        """
        from .version_sets_batch_delete.version_sets_batch_delete_request_builder import VersionSetsBatchDeleteRequestBuilder

        return VersionSetsBatchDeleteRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def version_sets_batch_get(self) -> VersionSetsBatchGetRequestBuilder:
        """
        The versionSetsBatchGet property
        """
        from .version_sets_batch_get.version_sets_batch_get_request_builder import VersionSetsBatchGetRequestBuilder

        return VersionSetsBatchGetRequestBuilder(self.request_adapter, self.path_parameters)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

@dataclass
class ScheduleOfValuesPostResponse(Parsable):
    # The total price of the SOV item.
    amount: Optional[float] = None
    # The ID of a change order or cost item that this SOV item was originally created from.
    association_id: Optional[str] = None
    # The type of object from which the SOV item was created:a change order or a cost item.
    association_type: Optional[str] = None
    # The ID of the budget to which the SOV item belongs.
    budget_id: Optional[str] = None
    # The converted quantity of the SOV item.
    bulk: Optional[float] = None
    # The unit price of the converted SOV item quantity.
    bulk_unit_price: Optional[float] = None
    # The code of the SOV item.Max length: 255
    code: Optional[str] = None
    # The ID of the contract to which the SOV item belongs.
    contract_id: Optional[str] = None
    # The date and time that the item was created, in ISO 8601 format.
    created_at: Optional[datetime.datetime] = None
    # The exchange rate that applies to the SOV item's base currency price. For example, provide the value ``0.7455`` for a foreign currency that's worth ``0.7455`` of your base currency.
    exchange_rate: Optional[float] = None
    # The identifier assigned to an item in its original external ERP system. Use this ID to track and look up data within the integrated system. Note that this value comes from the item's ID in the external system.Max length: 255
    external_id: Optional[str] = None
    # A message generated by the external ERP system that explains the sync status of the integration. For example, common values include ``success`` or ``fail`` to indicate the result of the integration operation.Max length: 255
    external_message: Optional[str] = None
    # The name of the external ERP system integrated with Cost Management. Use this name to identify and search for data within the integrated system.Max length: 255
    external_system: Optional[str] = None
    # The unique ID of this SOV item.
    id: Optional[UUID] = None
    # The date and time when the item was last synchronized with the external ERP system. This value is updated by the external system and is in ISO 8601 format.
    last_sync_time: Optional[datetime.datetime] = None
    # The name of the SOV item.Max length: 1024
    name: Optional[str] = None
    # The ID of the SOV item's parent item. For root SOV items, this value is **null**.Root items can only be created when allocating a budget to a contract.
    parent_id: Optional[str] = None
    # The position of the SOV item in the list of SOV items.
    position: Optional[float] = None
    # The quantity of the SOV item.
    quantity: Optional[float] = None
    # The quantity conversion ratio of the SOV item.
    quantity_per_bulk: Optional[float] = None
    # The unit of measure of the SOV item.Max length: 1024
    unit: Optional[str] = None
    # The unit price of the SOV item.
    unit_price: Optional[float] = None
    # The date and time that the item was last updated, in ISO 8601 format.
    updated_at: Optional[datetime.datetime] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ScheduleOfValuesPostResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ScheduleOfValuesPostResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ScheduleOfValuesPostResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "amount": lambda n : setattr(self, 'amount', n.get_float_value()),
            "associationId": lambda n : setattr(self, 'association_id', n.get_str_value()),
            "associationType": lambda n : setattr(self, 'association_type', n.get_str_value()),
            "budgetId": lambda n : setattr(self, 'budget_id', n.get_str_value()),
            "bulk": lambda n : setattr(self, 'bulk', n.get_float_value()),
            "bulkUnitPrice": lambda n : setattr(self, 'bulk_unit_price', n.get_float_value()),
            "code": lambda n : setattr(self, 'code', n.get_str_value()),
            "contractId": lambda n : setattr(self, 'contract_id', n.get_str_value()),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "exchangeRate": lambda n : setattr(self, 'exchange_rate', n.get_float_value()),
            "externalId": lambda n : setattr(self, 'external_id', n.get_str_value()),
            "externalMessage": lambda n : setattr(self, 'external_message', n.get_str_value()),
            "externalSystem": lambda n : setattr(self, 'external_system', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "lastSyncTime": lambda n : setattr(self, 'last_sync_time', n.get_datetime_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "parentId": lambda n : setattr(self, 'parent_id', n.get_str_value()),
            "position": lambda n : setattr(self, 'position', n.get_float_value()),
            "quantity": lambda n : setattr(self, 'quantity', n.get_float_value()),
            "quantityPerBulk": lambda n : setattr(self, 'quantity_per_bulk', n.get_float_value()),
            "unit": lambda n : setattr(self, 'unit', n.get_str_value()),
            "unitPrice": lambda n : setattr(self, 'unit_price', n.get_float_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_float_value("amount", self.amount)
        writer.write_str_value("associationId", self.association_id)
        writer.write_str_value("associationType", self.association_type)
        writer.write_str_value("budgetId", self.budget_id)
        writer.write_float_value("bulk", self.bulk)
        writer.write_float_value("bulkUnitPrice", self.bulk_unit_price)
        writer.write_str_value("code", self.code)
        writer.write_str_value("contractId", self.contract_id)
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_float_value("exchangeRate", self.exchange_rate)
        writer.write_str_value("externalId", self.external_id)
        writer.write_str_value("externalMessage", self.external_message)
        writer.write_str_value("externalSystem", self.external_system)
        writer.write_uuid_value("id", self.id)
        writer.write_datetime_value("lastSyncTime", self.last_sync_time)
        writer.write_str_value("name", self.name)
        writer.write_str_value("parentId", self.parent_id)
        writer.write_float_value("position", self.position)
        writer.write_float_value("quantity", self.quantity)
        writer.write_float_value("quantityPerBulk", self.quantity_per_bulk)
        writer.write_str_value("unit", self.unit)
        writer.write_float_value("unitPrice", self.unit_price)
        writer.write_datetime_value("updatedAt", self.updated_at)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .reviews_post_request_body_file_versions import ReviewsPostRequestBody_fileVersions
    from .reviews_post_request_body_workflow_options import ReviewsPostRequestBody_workflowOptions

@dataclass
class ReviewsPostRequestBody(Parsable):
    # The file versions to include in the review. Maximum: 1000 items.
    file_versions: Optional[list[ReviewsPostRequestBody_fileVersions]] = None
    # The name of the review. Maximum length: 255 characters.Max length: 255
    name: Optional[str] = None
    # A note about the review. In the UI, this appears as the ``Description`` field.Maximum length: 4096 characters.Max length: 4096
    notes: Optional[str] = None
    # The ID of the approval workflow used to create the review.To list available workflows, call `GET workflows </en/docs/acc/v1/reference/http/reviews-workflows-GET/>`_.
    workflow_id: Optional[UUID] = None
    # Optional parameters that override approval workflow settings for this review (for example, steps, copy settings, or additional options).
    workflow_options: Optional[list[ReviewsPostRequestBody_workflowOptions]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ReviewsPostRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ReviewsPostRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ReviewsPostRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .reviews_post_request_body_file_versions import ReviewsPostRequestBody_fileVersions
        from .reviews_post_request_body_workflow_options import ReviewsPostRequestBody_workflowOptions

        from .reviews_post_request_body_file_versions import ReviewsPostRequestBody_fileVersions
        from .reviews_post_request_body_workflow_options import ReviewsPostRequestBody_workflowOptions

        fields: dict[str, Callable[[Any], None]] = {
            "fileVersions": lambda n : setattr(self, 'file_versions', n.get_collection_of_object_values(ReviewsPostRequestBody_fileVersions)),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "notes": lambda n : setattr(self, 'notes', n.get_str_value()),
            "workflowId": lambda n : setattr(self, 'workflow_id', n.get_uuid_value()),
            "workflowOptions": lambda n : setattr(self, 'workflow_options', n.get_collection_of_object_values(ReviewsPostRequestBody_workflowOptions)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_object_values("fileVersions", self.file_versions)
        writer.write_str_value("name", self.name)
        writer.write_str_value("notes", self.notes)
        writer.write_uuid_value("workflowId", self.workflow_id)
        writer.write_collection_of_object_values("workflowOptions", self.workflow_options)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .attachments_post_request_body_attachments import AttachmentsPostRequestBody_attachments

@dataclass
class AttachmentsPostRequestBody(Parsable):
    # A list of attachments to add to the issue.
    attachments: Optional[list[AttachmentsPostRequestBody_attachments]] = None
    # The unique identifier of the issue to which the attachments will be added.
    domain_entity_id: Optional[UUID] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> AttachmentsPostRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: AttachmentsPostRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return AttachmentsPostRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .attachments_post_request_body_attachments import AttachmentsPostRequestBody_attachments

        from .attachments_post_request_body_attachments import AttachmentsPostRequestBody_attachments

        fields: dict[str, Callable[[Any], None]] = {
            "attachments": lambda n : setattr(self, 'attachments', n.get_collection_of_object_values(AttachmentsPostRequestBody_attachments)),
            "domainEntityId": lambda n : setattr(self, 'domain_entity_id', n.get_uuid_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_object_values("attachments", self.attachments)
        writer.write_uuid_value("domainEntityId", self.domain_entity_id)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .issue_attribute_definitions_get_response_results_metadata_list import IssueAttributeDefinitionsGetResponse_results_metadata_list

@dataclass
class IssueAttributeDefinitionsGetResponse_results_metadata(Parsable):
    """
    The metadata object; only relevant for ``list`` custom attributes.
    """
    # The list object.
    list_: Optional[IssueAttributeDefinitionsGetResponse_results_metadata_list] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> IssueAttributeDefinitionsGetResponse_results_metadata:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: IssueAttributeDefinitionsGetResponse_results_metadata
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return IssueAttributeDefinitionsGetResponse_results_metadata()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .issue_attribute_definitions_get_response_results_metadata_list import IssueAttributeDefinitionsGetResponse_results_metadata_list

        from .issue_attribute_definitions_get_response_results_metadata_list import IssueAttributeDefinitionsGetResponse_results_metadata_list

        fields: dict[str, Callable[[Any], None]] = {
            "list": lambda n : setattr(self, 'list_', n.get_object_value(IssueAttributeDefinitionsGetResponse_results_metadata_list)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_object_value("list", self.list_)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .sheets_get_response_results_collection import SheetsGetResponse_results_collection
    from .sheets_get_response_results_version_set import SheetsGetResponse_results_versionSet
    from .sheets_get_response_results_viewable import SheetsGetResponse_results_viewable

@dataclass
class SheetsGetResponse_results(Parsable):
    # The collection object, if assigned. If no collection is assigned, this value is ``null``.
    collection: Optional[SheetsGetResponse_results_collection] = None
    # The time when the sheet was created, in ISO-8601 format (YYYY-MM-DDTHH:mm:ss.SSSZ).
    created_at: Optional[datetime.datetime] = None
    # The ID of the user who created the sheet.
    created_by: Optional[str] = None
    # The name of the user who created the sheet.
    created_by_name: Optional[str] = None
    # - ``true`` if the sheet has been deleted.- ``false`` if the sheet has not been deleted.
    deleted: Optional[bool] = None
    # The time when the sheet was deleted, in ISO-8601 format (YYYY-MM-DDTHH:mm:ss.SSSZ).
    deleted_at: Optional[datetime.datetime] = None
    # The ID of the user who deleted the sheet.
    deleted_by: Optional[str] = None
    # The name of the user who deleted the sheet.
    deleted_by_name: Optional[str] = None
    # The ID of the sheet.
    id: Optional[UUID] = None
    # - ``true`` if the sheet is the version with the most recent issuance date. This is only relevant if you uploaded multiple versions of the same sheet that were assigned the same number. The ``current`` sheet is the sheet with the most recent issuance date.- ``false`` if the sheet is not the version with the most recent issuance date.
    is_current: Optional[bool] = None
    # The number of the sheet.
    number: Optional[str] = None
    # The size of the sheet in pixels.
    paper_size: Optional[list[float]] = None
    # The tags of the sheet.
    tags: Optional[list[str]] = None
    # The title of the sheet.
    title: Optional[str] = None
    # The time when the sheet was last updated, in ISO-8601 format (YYYY-MM-DDTHH:mm:ss.SSSZ).
    updated_at: Optional[datetime.datetime] = None
    # The ID of the user who last updated the sheet.
    updated_by: Optional[str] = None
    # The name of the user who last updated the sheet.
    updated_by_name: Optional[str] = None
    # The name of the source file from which the sheet was generated.
    upload_file_name: Optional[str] = None
    # The ID of the upload that generated the sheet.
    upload_id: Optional[UUID] = None
    # Basic version set data. For a complete collection of version set data, call `GET version-sets </en/docs/acc/v1/reference/http/sheets-version-sets-GET/>`_.
    version_set: Optional[SheetsGetResponse_results_versionSet] = None
    # Information about the sheet relevant for loading the sheet to the Viewer. See `Add Viewer to an HTML Page <https://forge.autodesk.com/en/docs/viewer/v7/developers_guide/viewer_basics/>`_ for more information.
    viewable: Optional[SheetsGetResponse_results_viewable] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> SheetsGetResponse_results:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: SheetsGetResponse_results
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return SheetsGetResponse_results()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .sheets_get_response_results_collection import SheetsGetResponse_results_collection
        from .sheets_get_response_results_version_set import SheetsGetResponse_results_versionSet
        from .sheets_get_response_results_viewable import SheetsGetResponse_results_viewable

        from .sheets_get_response_results_collection import SheetsGetResponse_results_collection
        from .sheets_get_response_results_version_set import SheetsGetResponse_results_versionSet
        from .sheets_get_response_results_viewable import SheetsGetResponse_results_viewable

        fields: dict[str, Callable[[Any], None]] = {
            "collection": lambda n : setattr(self, 'collection', n.get_object_value(SheetsGetResponse_results_collection)),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "createdBy": lambda n : setattr(self, 'created_by', n.get_str_value()),
            "createdByName": lambda n : setattr(self, 'created_by_name', n.get_str_value()),
            "deleted": lambda n : setattr(self, 'deleted', n.get_bool_value()),
            "deletedAt": lambda n : setattr(self, 'deleted_at', n.get_datetime_value()),
            "deletedBy": lambda n : setattr(self, 'deleted_by', n.get_str_value()),
            "deletedByName": lambda n : setattr(self, 'deleted_by_name', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "isCurrent": lambda n : setattr(self, 'is_current', n.get_bool_value()),
            "number": lambda n : setattr(self, 'number', n.get_str_value()),
            "paperSize": lambda n : setattr(self, 'paper_size', n.get_collection_of_primitive_values(float)),
            "tags": lambda n : setattr(self, 'tags', n.get_collection_of_primitive_values(str)),
            "title": lambda n : setattr(self, 'title', n.get_str_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
            "updatedBy": lambda n : setattr(self, 'updated_by', n.get_str_value()),
            "updatedByName": lambda n : setattr(self, 'updated_by_name', n.get_str_value()),
            "uploadFileName": lambda n : setattr(self, 'upload_file_name', n.get_str_value()),
            "uploadId": lambda n : setattr(self, 'upload_id', n.get_uuid_value()),
            "versionSet": lambda n : setattr(self, 'version_set', n.get_object_value(SheetsGetResponse_results_versionSet)),
            "viewable": lambda n : setattr(self, 'viewable', n.get_object_value(SheetsGetResponse_results_viewable)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_object_value("collection", self.collection)
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_str_value("createdBy", self.created_by)
        writer.write_str_value("createdByName", self.created_by_name)
        writer.write_bool_value("deleted", self.deleted)
        writer.write_datetime_value("deletedAt", self.deleted_at)
        writer.write_str_value("deletedBy", self.deleted_by)
        writer.write_str_value("deletedByName", self.deleted_by_name)
        writer.write_uuid_value("id", self.id)
        writer.write_bool_value("isCurrent", self.is_current)
        writer.write_str_value("number", self.number)
        writer.write_collection_of_primitive_values("paperSize", self.paper_size)
        writer.write_collection_of_primitive_values("tags", self.tags)
        writer.write_str_value("title", self.title)
        writer.write_datetime_value("updatedAt", self.updated_at)
        writer.write_str_value("updatedBy", self.updated_by)
        writer.write_str_value("updatedByName", self.updated_by_name)
        writer.write_str_value("uploadFileName", self.upload_file_name)
        writer.write_uuid_value("uploadId", self.upload_id)
        writer.write_object_value("versionSet", self.version_set)
        writer.write_object_value("viewable", self.viewable)
    




from enum import Enum

class TakeoffTypesGetResponse_results_shapeType(str, Enum):
    CIRCLE = "CIRCLE",
    TRIANGLE = "TRIANGLE",
    SQUARE = "SQUARE",
    DIAMOND = "DIAMOND",
    CHECKMARK = "CHECKMARK",




from enum import Enum

class WorkflowsGetResponse_results_steps_dueDateType(str, Enum):
    REVIEWER = "REVIEWER",
    APPROVER = "APPROVER",
    CALENDAR_DAY = "CALENDAR_DAY",
    WORKDAY = "WORKDAY",




from __future__ import annotations
from collections.abc import Callable
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .item.with_version_item_request_builder import WithVersion_ItemRequestBuilder

class VersionsRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /bim360/docs/v1/projects/{project-id}/versions
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new VersionsRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/bim360/docs/v1/projects/{project%2Did}/versions", path_parameters)
    
    def by_version_id(self,version_id: str) -> WithVersion_ItemRequestBuilder:
        """
        Gets an item from the Autodesk.ACC.bim360.docs.v1.projects.item.versions.item collection
        param version_id: The URL-encoded ID (URN) of the version. To find the version ID of a document follow the initial steps of the `Download Files </en/docs/bim360/v1/tutorials/document-management/download-document-s3/>`_ tutorial.
        Returns: WithVersion_ItemRequestBuilder
        """
        if version_id is None:
            raise TypeError("version_id cannot be null.")
        from .item.with_version_item_request_builder import WithVersion_ItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["version_id"] = version_id
        return WithVersion_ItemRequestBuilder(self.request_adapter, url_tpl_params)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class WithItemGetResponse_permittedActions_transitions_stateTo(Parsable):
    """
    The target state of the transition, indicating the next position of the submittal item in the workflow.
    """
    # The unique ID of the target state. For example, ``mgr-1``, ``mgr-2``, ``void``.
    id: Optional[str] = None
    # The name of the target state. For example, ``Manager Review``, ``Manager Final Review``, ``Void``.
    name: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithItemGetResponse_permittedActions_transitions_stateTo:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithItemGetResponse_permittedActions_transitions_stateTo
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithItemGetResponse_permittedActions_transitions_stateTo()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "id": lambda n : setattr(self, 'id', n.get_str_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("id", self.id)
        writer.write_str_value("name", self.name)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .change_order_patch_request_body_additional_collaborators import ChangeOrderPatchRequestBody_additionalCollaborators
    from .change_order_patch_request_body_scope import ChangeOrderPatchRequestBody_scope

@dataclass
class ChangeOrderPatchRequestBody(Parsable):
    # The additional collaborator company and contacts.
    additional_collaborators: Optional[list[ChangeOrderPatchRequestBody_additionalCollaborators]] = None
    # Not relevant
    approved: Optional[float] = None
    # The BIM360/ACC ID of the architecture firm.
    architect_company_id: Optional[str] = None
    # The unique ID (UUID) of the company in this account. Detailed company information can be retrieved using this UUID by calling `GET companies/:company_id <en/docs/bim360/v1/reference/http/companies-:company_id-GET/>`_ in the response.
    architect_company_uid: Optional[str] = None
    # The BIM360/ACC ID of the primary contact at the architecture firm.
    architect_contact_id: Optional[str] = None
    # Not relevant
    committed: Optional[float] = None
    # The BIM360/ACC ID of the company. For RFQ & SCO it's the supplier company. For OCO and OCO it's the owner company. For PCO, when PCO acts as RFQ it's the supplier company, when PCO asts as RCO it's the owner company, otherwise it's empty.
    company_id: Optional[str] = None
    # The unique ID (UUID) of the company in this account. Detailed company information can be retrieved using this UUID by calling `GET companies/:company_id <en/docs/bim360/v1/reference/http/companies-:company_id-GET/>`_ in the response.
    company_uid: Optional[str] = None
    # The description of the change order.Max length: 2048
    description: Optional[str] = None
    # Not relevant
    estimated: Optional[float] = None
    # A new currency exchange rate for the change order.Note that:- If the ``changeOrder`` parameter value is ``rfq``, this value updates the proposed exchange rate for all cost items under this RFQ.- If the ``changeOrder`` parameter value is ``sco``, this value updates the committed exchange rate for all cost items under this SCO.
    exchange_rate: Optional[float] = None
    # The identifier assigned to an item in its original external ERP system. Use this ID to track and look up data within the integrated system. Note that this value comes from the item's ID in the external system.Max length: 255
    external_id: Optional[str] = None
    # A message generated by the external ERP system that explains the sync status of the integration. For example, common values include ``success`` or ``fail`` to indicate the result of the integration operation.Max length: 255
    external_message: Optional[str] = None
    # The name of the external ERP system integrated with Cost Management. Use this name to identify and search for data within the integrated system.Max length: 255
    external_system: Optional[str] = None
    # The state of the item during the integration with the external ERP system (such as SignNow). An item can be a ``budget``, ``contract``, ``main contract``, ``main contract item``, ``cost item``, ``expense``, ``expense item``, ``change order``, or ``schedule of value``. For more details, see `Integrate with External System </en/docs/acc/v1/tutorials/cost/integrate-with-external-system/>`_ tutorial.Possible values:``locked``: the item is currently locked within the ERP system, preventing modifications until unlocked. To unlock and modify the item, use the relevant PATCH endpoint to set ``integrationState`` to ``null``. For example, for a budget, call `PATCH budgets <en/docs/bim360/v1/reference/http/cost-budgets-budgetId-PATCH/>`_. For a contract, call `PATCH contracts <en/docs/bim360/v1/reference/http/cost-contracts-contractId-PATCH/>`_. For more details, see the `Help documentation <https://help.autodesk.com/view/BUILD/ENU/?guid=Integrated_and_Locked>`_.``integrated``: the item has been successfully added to the ERP system.``failed``: the item encountered an error during the integration process and was not successfully added to the ERP system. For example, if a user tries to integrate ``contracts`` from an ERP system and the updates fail, the ``integrationState`` can be set to ``failed``. Retry the sync process or analyze the issue if it continues to fail.``null``: The item has not been integrated with the ERP system. This is default value.For more information regarding integrations within the Cost Management system, see `Integrations in Cost Management <https://help.autodesk.com/view/BUILD/ENU/?guid=Cost_Integrations>`_.
    integration_state: Optional[str] = None
    # The date and time when the item was last synchronized with the external ERP system. This value is updated by the external system and is in ISO 8601 format.
    last_sync_time: Optional[datetime.datetime] = None
    # The name of the change order.Max length: 1024
    name: Optional[str] = None
    # Additional notes to the change order. This is a Tiptap formatted rich text (https://tiptap.dev/introduction/).
    note: Optional[str] = None
    # The ID of the change order’s owner/purchaser, a project user managed by BIM 360 Admin.
    owner_id: Optional[str] = None
    # Not relevant
    proposed: Optional[float] = None
    # The proposed revised completion date of the change order.
    proposed_revised_completion_date: Optional[str] = None
    # Not relevant
    quantity: Optional[float] = None
    # The schedule change of the change order.
    schedule_change: Optional[float] = None
    # The scope of the change order. Possible values: ``out``, ``in``, ``tbd`` , ``budgetOnly``, ``contingency``.
    scope: Optional[ChangeOrderPatchRequestBody_scope] = None
    # The scope of work of the change order. This is a Tiptap formatted rich text (https://tiptap.dev/introduction/).
    scope_of_work: Optional[str] = None
    # The type of entity from which the change order is created (e.g., RFI, Issue).
    source_type: Optional[str] = None
    # Not relevant
    submitted: Optional[float] = None
    # The type of the change order. It is customizable by the project admin.
    type: Optional[str] = None
    # Not relevant
    unit: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ChangeOrderPatchRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ChangeOrderPatchRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ChangeOrderPatchRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .change_order_patch_request_body_additional_collaborators import ChangeOrderPatchRequestBody_additionalCollaborators
        from .change_order_patch_request_body_scope import ChangeOrderPatchRequestBody_scope

        from .change_order_patch_request_body_additional_collaborators import ChangeOrderPatchRequestBody_additionalCollaborators
        from .change_order_patch_request_body_scope import ChangeOrderPatchRequestBody_scope

        fields: dict[str, Callable[[Any], None]] = {
            "additionalCollaborators": lambda n : setattr(self, 'additional_collaborators', n.get_collection_of_object_values(ChangeOrderPatchRequestBody_additionalCollaborators)),
            "approved": lambda n : setattr(self, 'approved', n.get_float_value()),
            "architectCompanyId": lambda n : setattr(self, 'architect_company_id', n.get_str_value()),
            "architectCompanyUid": lambda n : setattr(self, 'architect_company_uid', n.get_str_value()),
            "architectContactId": lambda n : setattr(self, 'architect_contact_id', n.get_str_value()),
            "committed": lambda n : setattr(self, 'committed', n.get_float_value()),
            "companyId": lambda n : setattr(self, 'company_id', n.get_str_value()),
            "companyUid": lambda n : setattr(self, 'company_uid', n.get_str_value()),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "estimated": lambda n : setattr(self, 'estimated', n.get_float_value()),
            "exchangeRate": lambda n : setattr(self, 'exchange_rate', n.get_float_value()),
            "externalId": lambda n : setattr(self, 'external_id', n.get_str_value()),
            "externalMessage": lambda n : setattr(self, 'external_message', n.get_str_value()),
            "externalSystem": lambda n : setattr(self, 'external_system', n.get_str_value()),
            "integrationState": lambda n : setattr(self, 'integration_state', n.get_str_value()),
            "lastSyncTime": lambda n : setattr(self, 'last_sync_time', n.get_datetime_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "note": lambda n : setattr(self, 'note', n.get_str_value()),
            "ownerId": lambda n : setattr(self, 'owner_id', n.get_str_value()),
            "proposed": lambda n : setattr(self, 'proposed', n.get_float_value()),
            "proposedRevisedCompletionDate": lambda n : setattr(self, 'proposed_revised_completion_date', n.get_str_value()),
            "quantity": lambda n : setattr(self, 'quantity', n.get_float_value()),
            "scheduleChange": lambda n : setattr(self, 'schedule_change', n.get_float_value()),
            "scope": lambda n : setattr(self, 'scope', n.get_enum_value(ChangeOrderPatchRequestBody_scope)),
            "scopeOfWork": lambda n : setattr(self, 'scope_of_work', n.get_str_value()),
            "sourceType": lambda n : setattr(self, 'source_type', n.get_str_value()),
            "submitted": lambda n : setattr(self, 'submitted', n.get_float_value()),
            "type": lambda n : setattr(self, 'type', n.get_str_value()),
            "unit": lambda n : setattr(self, 'unit', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_object_values("additionalCollaborators", self.additional_collaborators)
        writer.write_float_value("approved", self.approved)
        writer.write_str_value("architectCompanyId", self.architect_company_id)
        writer.write_str_value("architectCompanyUid", self.architect_company_uid)
        writer.write_str_value("architectContactId", self.architect_contact_id)
        writer.write_float_value("committed", self.committed)
        writer.write_str_value("companyId", self.company_id)
        writer.write_str_value("companyUid", self.company_uid)
        writer.write_str_value("description", self.description)
        writer.write_float_value("estimated", self.estimated)
        writer.write_float_value("exchangeRate", self.exchange_rate)
        writer.write_str_value("externalId", self.external_id)
        writer.write_str_value("externalMessage", self.external_message)
        writer.write_str_value("externalSystem", self.external_system)
        writer.write_str_value("integrationState", self.integration_state)
        writer.write_datetime_value("lastSyncTime", self.last_sync_time)
        writer.write_str_value("name", self.name)
        writer.write_str_value("note", self.note)
        writer.write_str_value("ownerId", self.owner_id)
        writer.write_float_value("proposed", self.proposed)
        writer.write_str_value("proposedRevisedCompletionDate", self.proposed_revised_completion_date)
        writer.write_float_value("quantity", self.quantity)
        writer.write_float_value("scheduleChange", self.schedule_change)
        writer.write_enum_value("scope", self.scope)
        writer.write_str_value("scopeOfWork", self.scope_of_work)
        writer.write_str_value("sourceType", self.source_type)
        writer.write_float_value("submitted", self.submitted)
        writer.write_str_value("type", self.type)
        writer.write_str_value("unit", self.unit)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class IssuesPostResponse_linkedDocuments_details_viewable(Parsable):
    """
    The individual viewable associated with the issue (pushpin). This is relevant for both individual 2D sheets and views within a 3D model, and individual PDF sheets within a multi-sheet PDF file. It is only relevant if the issue is associated with a file.
    """
    # The ID of the viewable (guid).
    guid: Optional[str] = None
    # Not relevant
    id: Optional[str] = None
    # ``true`` if it is a 3D viewable``false`` if it is a 2D viewable
    is3_d: Optional[bool] = None
    # The name of the viewable.Max length: 1000
    name: Optional[str] = None
    # Not relevant
    viewable_id: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> IssuesPostResponse_linkedDocuments_details_viewable:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: IssuesPostResponse_linkedDocuments_details_viewable
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return IssuesPostResponse_linkedDocuments_details_viewable()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "guid": lambda n : setattr(self, 'guid', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_str_value()),
            "is3D": lambda n : setattr(self, 'is3_d', n.get_bool_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "viewableId": lambda n : setattr(self, 'viewable_id', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("guid", self.guid)
        writer.write_str_value("id", self.id)
        writer.write_bool_value("is3D", self.is3_d)
        writer.write_str_value("name", self.name)
        writer.write_str_value("viewableId", self.viewable_id)
    




from enum import Enum

class WithUploadGetResponse_status(str, Enum):
    PENDING = "PENDING",
    PROCESSING = "PROCESSING",
    IN_REVIEW = "IN_REVIEW",
    FAILED = "FAILED",
    UPDATING_VERSION_SET = "UPDATING_VERSION_SET",
    PUBLISHING = "PUBLISHING",
    PUBLISHED = "PUBLISHED",




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class CategoriesBatchGetPostResponse_results(Parsable):
    # The time when the component was created (ISO8601 Date time format in UTC).
    created_at: Optional[str] = None
    # The actor that created the component. This is an Autodesk / Oxygen ID.
    created_by: Optional[str] = None
    # The time when the component was deleted at (ISO8601 Date time format in UTC).
    deleted_at: Optional[str] = None
    # The actor that deleted the component. This is an Autodesk / Oxygen ID.
    deleted_by: Optional[str] = None
    # A description of the category.
    description: Optional[str] = None
    # The ID of the category. This is a numeric string for ``v1`` category APIs.
    id: Optional[str] = None
    # A flag indicating whether the component is active or inactive (``isActive`` is ``true`` if-and-only-if ``deletedAt`` is empty).
    is_active: Optional[bool] = None
    # Whether or not this category is a leaf category of the category tree. If ``true``, it's a leaf; if ``false``,it's not.Note that this is a derived field and should not be persisted as this field may be updated withoutupdating the category itself.
    is_leaf: Optional[bool] = None
    # Whether or not this category is the root category of the category tree. If ``true``, it's the root; if``false``, it's not. There will only ever be one (immutable) root category for a project, which is createdautomatically when the project is created.
    is_root: Optional[bool] = None
    # The name of the category. Must be unique among children of the same parent category. This name is displayedfor the category in the Assets user interface where a user chooses categories.
    name: Optional[str] = None
    # The ID of the category's parent category.
    parent_id: Optional[str] = None
    # An array of category IDs of this category's child categories.Note that this is a derived field and should not be persisted as this field may be updated withoutupdating the category itself. As such, it is highly recommended to use the ``parentId`` field toconstruct the tree locally instead of ``subcategoryIds``.
    subcategory_ids: Optional[list[str]] = None
    # The globally-unique ID of a ``Category`` from the ``v3`` (upcoming) Category APIs. This is providedfor interoperability with other APIs that use the globally-unique category ID. Only included if the``includeUid=true`` query param is included.
    uid: Optional[str] = None
    # The time when the component was last updated (ISO8601 Date time format in UTC).
    updated_at: Optional[str] = None
    # The actor that last updated the component. This is an Autodesk / Oxygen ID.
    updated_by: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> CategoriesBatchGetPostResponse_results:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: CategoriesBatchGetPostResponse_results
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return CategoriesBatchGetPostResponse_results()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "createdAt": lambda n : setattr(self, 'created_at', n.get_str_value()),
            "createdBy": lambda n : setattr(self, 'created_by', n.get_str_value()),
            "deletedAt": lambda n : setattr(self, 'deleted_at', n.get_str_value()),
            "deletedBy": lambda n : setattr(self, 'deleted_by', n.get_str_value()),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_str_value()),
            "isActive": lambda n : setattr(self, 'is_active', n.get_bool_value()),
            "isLeaf": lambda n : setattr(self, 'is_leaf', n.get_bool_value()),
            "isRoot": lambda n : setattr(self, 'is_root', n.get_bool_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "parentId": lambda n : setattr(self, 'parent_id', n.get_str_value()),
            "subcategoryIds": lambda n : setattr(self, 'subcategory_ids', n.get_collection_of_primitive_values(str)),
            "uid": lambda n : setattr(self, 'uid', n.get_str_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_str_value()),
            "updatedBy": lambda n : setattr(self, 'updated_by', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("createdAt", self.created_at)
        writer.write_str_value("createdBy", self.created_by)
        writer.write_str_value("deletedAt", self.deleted_at)
        writer.write_str_value("deletedBy", self.deleted_by)
        writer.write_str_value("description", self.description)
        writer.write_str_value("id", self.id)
        writer.write_bool_value("isActive", self.is_active)
        writer.write_bool_value("isLeaf", self.is_leaf)
        writer.write_bool_value("isRoot", self.is_root)
        writer.write_str_value("name", self.name)
        writer.write_str_value("parentId", self.parent_id)
        writer.write_collection_of_primitive_values("subcategoryIds", self.subcategory_ids)
        writer.write_str_value("uid", self.uid)
        writer.write_str_value("updatedAt", self.updated_at)
        writer.write_str_value("updatedBy", self.updated_by)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

@dataclass
class WithPackageGetResponse(Parsable):
    # The date and time when the resource was created, in the following format: ``YYYY-MM-DDThh:mm:ssZ``.
    created_at: Optional[datetime.datetime] = None
    # The package ID.
    id: Optional[UUID] = None
    # The package name (user defined).Corresponding UI name: ``Title``.Max length: 64
    name: Optional[str] = None
    # The date and time when the resource was last updated, in the following format: ``YYYY-MM-DDThh:mm:ssZ``.
    updated_at: Optional[datetime.datetime] = None
    # The name of the user who last updated the resource.
    updated_by_name: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithPackageGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithPackageGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithPackageGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
            "updatedByName": lambda n : setattr(self, 'updated_by_name', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_uuid_value("id", self.id)
        writer.write_str_value("name", self.name)
        writer.write_datetime_value("updatedAt", self.updated_at)
        writer.write_str_value("updatedByName", self.updated_by_name)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .metadata_get_response_default_values_watchers import MetadataGetResponse_defaultValues_watchers

@dataclass
class MetadataGetResponse_defaultValues(Parsable):
    # The Autodesk ID of the user who is assigned as the manager.
    manager: Optional[str] = None
    # The number of days remaining until the review due date.
    review_time: Optional[int] = None
    # The date and time when the response was last updated, in ISO 8601 format.
    updated_at: Optional[datetime.datetime] = None
    # The Autodesk ID of the user that last updated the response.
    updated_by: Optional[str] = None
    # A list of project watchers, who can be individual users, roles, or companies.
    watchers: Optional[list[MetadataGetResponse_defaultValues_watchers]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> MetadataGetResponse_defaultValues:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: MetadataGetResponse_defaultValues
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return MetadataGetResponse_defaultValues()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .metadata_get_response_default_values_watchers import MetadataGetResponse_defaultValues_watchers

        from .metadata_get_response_default_values_watchers import MetadataGetResponse_defaultValues_watchers

        fields: dict[str, Callable[[Any], None]] = {
            "manager": lambda n : setattr(self, 'manager', n.get_str_value()),
            "reviewTime": lambda n : setattr(self, 'review_time', n.get_int_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
            "updatedBy": lambda n : setattr(self, 'updated_by', n.get_str_value()),
            "watchers": lambda n : setattr(self, 'watchers', n.get_collection_of_object_values(MetadataGetResponse_defaultValues_watchers)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("manager", self.manager)
        writer.write_int_value("reviewTime", self.review_time)
        writer.write_datetime_value("updatedAt", self.updated_at)
        writer.write_str_value("updatedBy", self.updated_by)
        writer.write_collection_of_object_values("watchers", self.watchers)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .issues_get_response_results_linked_documents_details import IssuesGetResponse_results_linkedDocuments_details
    from .issues_get_response_results_linked_documents_type import IssuesGetResponse_results_linkedDocuments_type

@dataclass
class IssuesGetResponse_results_linkedDocuments(Parsable):
    # The date and time the pushpin issue was closed, in ISO8601 format.
    closed_at: Optional[datetime.datetime] = None
    # The version of the file when the pushpin issue was closed.
    closed_at_version: Optional[int] = None
    # The Autodesk ID of the user who closed the pushpin issue.
    closed_by: Optional[str] = None
    # The date and time the pushpin was created, in ISO8601 format.
    created_at: Optional[datetime.datetime] = None
    # The version of the file the pushin issue was added to. For information about file versions, see the `Data Management API </en/docs/data/v2/>`_.
    created_at_version: Optional[int] = None
    # The Autodesk ID of the user who created the pushpin issue.
    created_by: Optional[str] = None
    # Information about the individual viewable.
    details: Optional[IssuesGetResponse_results_linkedDocuments_details] = None
    # The type of file. Possible values:``TwoDVectorPushpin`` (3D models)``TwoDRasterPushpin`` (2D sheets and views)
    type: Optional[IssuesGetResponse_results_linkedDocuments_type] = None
    # The ID of the file associated with the issue (pushpin). Note that we do not currently support data associated with the ACC Build Sheet tool.
    urn: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> IssuesGetResponse_results_linkedDocuments:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: IssuesGetResponse_results_linkedDocuments
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return IssuesGetResponse_results_linkedDocuments()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .issues_get_response_results_linked_documents_details import IssuesGetResponse_results_linkedDocuments_details
        from .issues_get_response_results_linked_documents_type import IssuesGetResponse_results_linkedDocuments_type

        from .issues_get_response_results_linked_documents_details import IssuesGetResponse_results_linkedDocuments_details
        from .issues_get_response_results_linked_documents_type import IssuesGetResponse_results_linkedDocuments_type

        fields: dict[str, Callable[[Any], None]] = {
            "closedAt": lambda n : setattr(self, 'closed_at', n.get_datetime_value()),
            "closedAtVersion": lambda n : setattr(self, 'closed_at_version', n.get_int_value()),
            "closedBy": lambda n : setattr(self, 'closed_by', n.get_str_value()),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "createdAtVersion": lambda n : setattr(self, 'created_at_version', n.get_int_value()),
            "createdBy": lambda n : setattr(self, 'created_by', n.get_str_value()),
            "details": lambda n : setattr(self, 'details', n.get_object_value(IssuesGetResponse_results_linkedDocuments_details)),
            "type": lambda n : setattr(self, 'type', n.get_enum_value(IssuesGetResponse_results_linkedDocuments_type)),
            "urn": lambda n : setattr(self, 'urn', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_datetime_value("closedAt", self.closed_at)
        writer.write_int_value("closedAtVersion", self.closed_at_version)
        writer.write_str_value("closedBy", self.closed_by)
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_int_value("createdAtVersion", self.created_at_version)
        writer.write_str_value("createdBy", self.created_by)
        writer.write_object_value("details", self.details)
        writer.write_enum_value("type", self.type)
        writer.write_str_value("urn", self.urn)
    




from __future__ import annotations
from collections.abc import Callable
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .relationships.relationships_request_builder import RelationshipsRequestBuilder
    from .relationships_batch.relationships_batch_request_builder import RelationshipsBatchRequestBuilder
    from .relationships_delete.relationships_delete_request_builder import RelationshipsDeleteRequestBuilder
    from .relationships_intersect.relationships_intersect_request_builder import RelationshipsIntersectRequestBuilder
    from .relationships_search.relationships_search_request_builder import RelationshipsSearchRequestBuilder
    from .relationships_sync.relationships_sync_request_builder import RelationshipsSyncRequestBuilder
    from .relationships_sync_status.relationships_sync_status_request_builder import RelationshipsSyncStatusRequestBuilder

class WithContainerItemRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /bim360/relationship/v2/containers/{containerId}
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new WithContainerItemRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/bim360/relationship/v2/containers/{containerId}", path_parameters)
    
    @property
    def relationships(self) -> RelationshipsRequestBuilder:
        """
        The relationships property
        """
        from .relationships.relationships_request_builder import RelationshipsRequestBuilder

        return RelationshipsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def relationships_batch(self) -> RelationshipsBatchRequestBuilder:
        """
        The relationshipsBatch property
        """
        from .relationships_batch.relationships_batch_request_builder import RelationshipsBatchRequestBuilder

        return RelationshipsBatchRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def relationships_delete(self) -> RelationshipsDeleteRequestBuilder:
        """
        The relationshipsDelete property
        """
        from .relationships_delete.relationships_delete_request_builder import RelationshipsDeleteRequestBuilder

        return RelationshipsDeleteRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def relationships_intersect(self) -> RelationshipsIntersectRequestBuilder:
        """
        The relationshipsIntersect property
        """
        from .relationships_intersect.relationships_intersect_request_builder import RelationshipsIntersectRequestBuilder

        return RelationshipsIntersectRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def relationships_search(self) -> RelationshipsSearchRequestBuilder:
        """
        The relationshipsSearch property
        """
        from .relationships_search.relationships_search_request_builder import RelationshipsSearchRequestBuilder

        return RelationshipsSearchRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def relationships_sync(self) -> RelationshipsSyncRequestBuilder:
        """
        The relationshipsSync property
        """
        from .relationships_sync.relationships_sync_request_builder import RelationshipsSyncRequestBuilder

        return RelationshipsSyncRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def relationships_sync_status(self) -> RelationshipsSyncStatusRequestBuilder:
        """
        The relationshipsSyncStatus property
        """
        from .relationships_sync_status.relationships_sync_status_request_builder import RelationshipsSyncStatusRequestBuilder

        return RelationshipsSyncStatusRequestBuilder(self.request_adapter, self.path_parameters)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .with_takeoff_type_get_response_property_definitions_unit_of_measure import WithTakeoffTypeGetResponse_propertyDefinitions_unitOfMeasure
    from .with_takeoff_type_get_response_property_definitions_value_location import WithTakeoffTypeGetResponse_propertyDefinitions_valueLocation

@dataclass
class WithTakeoffTypeGetResponse_propertyDefinitions(Parsable):
    # The name of the property.Max length: 256
    name: Optional[str] = None
    # The unit of measurement.Possible values: ``EA``, ``IN``, ``LF``, ``YD``, ``SI``, ``SF``, ``SY``, ``CI``, ``CF``, ``CY``, ``LBS``, ``TON``, ``MM``, ``M``, ``M2``, ``M3``, ``KG``, ``T``.
    unit_of_measure: Optional[WithTakeoffTypeGetResponse_propertyDefinitions_unitOfMeasure] = None
    # The value of the property.
    value: Optional[float] = None
    # The location of the value.For properties Length, Width, Height, Depth and Thickness: ``valueLocation`` is ``INSTANCE_WITH_TAKEOFF_TYPE_DEFAULT``.For properties WeightByLength, WeightByArea and WeightByVolume: ``valueLocation`` is ``TAKEOFF_TYPE``.Possible values: ``TAKEOFF_TYPE``, ``INSTANCE_WITH_TAKEOFF_TYPE_DEFAULT``
    value_location: Optional[WithTakeoffTypeGetResponse_propertyDefinitions_valueLocation] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithTakeoffTypeGetResponse_propertyDefinitions:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithTakeoffTypeGetResponse_propertyDefinitions
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithTakeoffTypeGetResponse_propertyDefinitions()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .with_takeoff_type_get_response_property_definitions_unit_of_measure import WithTakeoffTypeGetResponse_propertyDefinitions_unitOfMeasure
        from .with_takeoff_type_get_response_property_definitions_value_location import WithTakeoffTypeGetResponse_propertyDefinitions_valueLocation

        from .with_takeoff_type_get_response_property_definitions_unit_of_measure import WithTakeoffTypeGetResponse_propertyDefinitions_unitOfMeasure
        from .with_takeoff_type_get_response_property_definitions_value_location import WithTakeoffTypeGetResponse_propertyDefinitions_valueLocation

        fields: dict[str, Callable[[Any], None]] = {
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "unitOfMeasure": lambda n : setattr(self, 'unit_of_measure', n.get_enum_value(WithTakeoffTypeGetResponse_propertyDefinitions_unitOfMeasure)),
            "value": lambda n : setattr(self, 'value', n.get_float_value()),
            "valueLocation": lambda n : setattr(self, 'value_location', n.get_enum_value(WithTakeoffTypeGetResponse_propertyDefinitions_valueLocation)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("name", self.name)
        writer.write_enum_value("unitOfMeasure", self.unit_of_measure)
        writer.write_float_value("value", self.value)
        writer.write_enum_value("valueLocation", self.value_location)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .workflows_post_request_body_additional_approval_status_options_value import WorkflowsPostRequestBody_additionalApprovalStatusOptions_value

@dataclass
class WorkflowsPostRequestBody_additionalApprovalStatusOptions(Parsable):
    # The display name shown in the UI. It m ust be unique across all status options (built-in and custom).Maximum length: 255 characters.Max length: 255
    label: Optional[str] = None
    # The value representing the approval outcome. Possible values: ``APPROVED``, ``REJECTED``.
    value: Optional[WorkflowsPostRequestBody_additionalApprovalStatusOptions_value] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WorkflowsPostRequestBody_additionalApprovalStatusOptions:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WorkflowsPostRequestBody_additionalApprovalStatusOptions
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WorkflowsPostRequestBody_additionalApprovalStatusOptions()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .workflows_post_request_body_additional_approval_status_options_value import WorkflowsPostRequestBody_additionalApprovalStatusOptions_value

        from .workflows_post_request_body_additional_approval_status_options_value import WorkflowsPostRequestBody_additionalApprovalStatusOptions_value

        fields: dict[str, Callable[[Any], None]] = {
            "label": lambda n : setattr(self, 'label', n.get_str_value()),
            "value": lambda n : setattr(self, 'value', n.get_enum_value(WorkflowsPostRequestBody_additionalApprovalStatusOptions_value)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("label", self.label)
        writer.write_enum_value("value", self.value)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .users_get_response_results_access_levels import UsersGetResponse_results_accessLevels
    from .users_get_response_results_phone import UsersGetResponse_results_phone
    from .users_get_response_results_products import UsersGetResponse_results_products
    from .users_get_response_results_roles import UsersGetResponse_results_roles

@dataclass
class UsersGetResponse_results(Parsable):
    # A short bio about the user. This data syncs from the user's Autodesk profile.Max length: 255
    about_me: Optional[str] = None
    # Flags that identify a returned user's access levels in the account or project.
    access_levels: Optional[UsersGetResponse_results_accessLevels] = None
    # The timestamp when the user was first given access to any product on the project.
    added_on: Optional[datetime.datetime] = None
    # The user's address line 1. This data syncs from the user's Autodesk profile.Max length: 255
    address_line1: Optional[str] = None
    # The user's address line 2. This data syncs from the user's Autodesk profile.Max length: 255
    address_line2: Optional[str] = None
    # Not relevant
    analytics_id: Optional[str] = None
    # The ID of the user's Autodesk profile.Max length: 255
    autodesk_id: Optional[str] = None
    # The User's city. This data syncs from the user's Autodesk profile.Max length: 255
    city: Optional[str] = None
    # The ID of the company that the user is representing in the project. To obtain a list of all company IDs associated with a project, call `GET projects/:projectId/companies </en/docs/acc/v1/reference/http/projects-:project_id-companies-GET/>`_.
    company_id: Optional[str] = None
    # The name of the company to which the user belongs.Max length: 255
    company_name: Optional[str] = None
    # The user's country. This data syncs from the user's Autodesk profile.Max length: 255
    country: Optional[str] = None
    # The email of the user.Max length: 255
    email: Optional[str] = None
    # The user's first name. This data syncs from the user's Autodesk profile.Max length: 255
    first_name: Optional[str] = None
    # The ACC ID of the user.
    id: Optional[UUID] = None
    # The URL of the user's avatar. This data syncs from the user's Autodesk profile.Max length: 255
    image_url: Optional[str] = None
    # The industry the user works in. This data syncs from the user's Autodesk profile.Max length: 255
    industry: Optional[str] = None
    # The user's job title. This data syncs from the user's Autodesk profile.Max length: 255
    job_title: Optional[str] = None
    # The user's last name. This data syncs from the user's Autodesk profile.Max length: 255
    last_name: Optional[str] = None
    # The full name of the user.Max length: 255
    name: Optional[str] = None
    # The user's phone number. This data syncs from the user's Autodesk profile.
    phone: Optional[UsersGetResponse_results_phone] = None
    # The zip or postal code of the user. This data syncs from the user's Autodesk profile.Max length: 255
    postal_code: Optional[str] = None
    # Information about the products activated in the specified project for this user.
    products: Optional[list[UsersGetResponse_results_products]] = None
    # A list of IDs of the roles that the user belongs to in the project.
    role_ids: Optional[list[str]] = None
    # A list of the role IDs and names that are associated with the user in the project.
    roles: Optional[list[UsersGetResponse_results_roles]] = None
    # The state or province of the user. The accepted values here change depending on which country is provided. This data syncs from the user's Autodesk profile.Max length: 255
    state_or_province: Optional[str] = None
    # The status of the user in the project. A pending user could be waiting for their products to activate, or the user hasn't accepted an email to create an account with Autodesk.Possible values:- ``active``: The user has been added to the project.- ``pending``: The user is in the process of being added to the project.- ``disabled``: The user has been temporarily suspended from the project.- ``deleted``: The user has been removed from the project.
    status: Optional[str] = None
    # The timestamp when the project user was last updated, in ISO 8601 format.
    updated_at: Optional[datetime.datetime] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> UsersGetResponse_results:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: UsersGetResponse_results
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return UsersGetResponse_results()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .users_get_response_results_access_levels import UsersGetResponse_results_accessLevels
        from .users_get_response_results_phone import UsersGetResponse_results_phone
        from .users_get_response_results_products import UsersGetResponse_results_products
        from .users_get_response_results_roles import UsersGetResponse_results_roles

        from .users_get_response_results_access_levels import UsersGetResponse_results_accessLevels
        from .users_get_response_results_phone import UsersGetResponse_results_phone
        from .users_get_response_results_products import UsersGetResponse_results_products
        from .users_get_response_results_roles import UsersGetResponse_results_roles

        fields: dict[str, Callable[[Any], None]] = {
            "aboutMe": lambda n : setattr(self, 'about_me', n.get_str_value()),
            "accessLevels": lambda n : setattr(self, 'access_levels', n.get_object_value(UsersGetResponse_results_accessLevels)),
            "addedOn": lambda n : setattr(self, 'added_on', n.get_datetime_value()),
            "addressLine1": lambda n : setattr(self, 'address_line1', n.get_str_value()),
            "addressLine2": lambda n : setattr(self, 'address_line2', n.get_str_value()),
            "analyticsId": lambda n : setattr(self, 'analytics_id', n.get_str_value()),
            "autodeskId": lambda n : setattr(self, 'autodesk_id', n.get_str_value()),
            "city": lambda n : setattr(self, 'city', n.get_str_value()),
            "companyId": lambda n : setattr(self, 'company_id', n.get_str_value()),
            "companyName": lambda n : setattr(self, 'company_name', n.get_str_value()),
            "country": lambda n : setattr(self, 'country', n.get_str_value()),
            "email": lambda n : setattr(self, 'email', n.get_str_value()),
            "firstName": lambda n : setattr(self, 'first_name', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "imageUrl": lambda n : setattr(self, 'image_url', n.get_str_value()),
            "industry": lambda n : setattr(self, 'industry', n.get_str_value()),
            "jobTitle": lambda n : setattr(self, 'job_title', n.get_str_value()),
            "lastName": lambda n : setattr(self, 'last_name', n.get_str_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "phone": lambda n : setattr(self, 'phone', n.get_object_value(UsersGetResponse_results_phone)),
            "postalCode": lambda n : setattr(self, 'postal_code', n.get_str_value()),
            "products": lambda n : setattr(self, 'products', n.get_collection_of_object_values(UsersGetResponse_results_products)),
            "roleIds": lambda n : setattr(self, 'role_ids', n.get_collection_of_primitive_values(str)),
            "roles": lambda n : setattr(self, 'roles', n.get_collection_of_object_values(UsersGetResponse_results_roles)),
            "stateOrProvince": lambda n : setattr(self, 'state_or_province', n.get_str_value()),
            "status": lambda n : setattr(self, 'status', n.get_str_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("aboutMe", self.about_me)
        writer.write_object_value("accessLevels", self.access_levels)
        writer.write_datetime_value("addedOn", self.added_on)
        writer.write_str_value("addressLine1", self.address_line1)
        writer.write_str_value("addressLine2", self.address_line2)
        writer.write_str_value("analyticsId", self.analytics_id)
        writer.write_str_value("autodeskId", self.autodesk_id)
        writer.write_str_value("city", self.city)
        writer.write_str_value("companyId", self.company_id)
        writer.write_str_value("companyName", self.company_name)
        writer.write_str_value("country", self.country)
        writer.write_str_value("email", self.email)
        writer.write_str_value("firstName", self.first_name)
        writer.write_uuid_value("id", self.id)
        writer.write_str_value("imageUrl", self.image_url)
        writer.write_str_value("industry", self.industry)
        writer.write_str_value("jobTitle", self.job_title)
        writer.write_str_value("lastName", self.last_name)
        writer.write_str_value("name", self.name)
        writer.write_object_value("phone", self.phone)
        writer.write_str_value("postalCode", self.postal_code)
        writer.write_collection_of_object_values("products", self.products)
        writer.write_collection_of_primitive_values("roleIds", self.role_ids)
        writer.write_collection_of_object_values("roles", self.roles)
        writer.write_str_value("stateOrProvince", self.state_or_province)
        writer.write_str_value("status", self.status)
        writer.write_datetime_value("updatedAt", self.updated_at)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from .form_templates_get_response import FormTemplatesGetResponse
    from .get_sort_order_query_parameter_type import GetSortOrderQueryParameterType
    from .item.with_template_item_request_builder import WithTemplateItemRequestBuilder

class FormTemplatesRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/forms/v1/projects/{projectId}/form-templates
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new FormTemplatesRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/forms/v1/projects/{projectId}/form-templates{?limit*,offset*,sortOrder*,updatedAfter*,updatedBefore*}", path_parameters)
    
    def by_template_id(self,template_id: str) -> WithTemplateItemRequestBuilder:
        """
        Gets an item from the Autodesk.ACC.construction.forms.v1.projects.item.formTemplates.item collection
        param template_id: The unique identifier of the form template.Use `GET form-templates </en/docs/acc/v1/reference/http/forms-form-templates-GET/>`_ to retrieve the template ID.
        Returns: WithTemplateItemRequestBuilder
        """
        if template_id is None:
            raise TypeError("template_id cannot be null.")
        from .item.with_template_item_request_builder import WithTemplateItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["templateId"] = template_id
        return WithTemplateItemRequestBuilder(self.request_adapter, url_tpl_params)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[FormTemplatesRequestBuilderGetQueryParameters]] = None) -> Optional[FormTemplatesGetResponse]:
        """
        Returns all project's form templates the user has access to.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[FormTemplatesGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .form_templates_get_response import FormTemplatesGetResponse

        return await self.request_adapter.send_async(request_info, FormTemplatesGetResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[FormTemplatesRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        Returns all project's form templates the user has access to.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def with_url(self,raw_url: str) -> FormTemplatesRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: FormTemplatesRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return FormTemplatesRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class FormTemplatesRequestBuilderGetQueryParameters():
        """
        Returns all project's form templates the user has access to.
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "sort_order":
                return "sortOrder"
            if original_name == "updated_after":
                return "updatedAfter"
            if original_name == "updated_before":
                return "updatedBefore"
            if original_name == "limit":
                return "limit"
            if original_name == "offset":
                return "offset"
            return original_name
        
        # The number of records to return in a single request. Can be a number between 1 and 50. Defaults to 50.
        limit: Optional[int] = None

        # The number of records to skip before returning the result records. Defaults to 0. Increase this value in subsequent requests to continue getting results when the number of records exceeds the requested limit.
        offset: Optional[int] = None

        # Return Templates in specified sorted order.Possible values: ``desc``, ``asc``
        sort_order: Optional[GetSortOrderQueryParameterType] = None

        # Return Templates updated after specified time.
        updated_after: Optional[datetime.datetime] = None

        # Return Templates updated before specified time.
        updated_before: Optional[datetime.datetime] = None

    
    @dataclass
    class FormTemplatesRequestBuilderGetRequestConfiguration(RequestConfiguration[FormTemplatesRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from enum import Enum

class CustomAttributesBatchGetPostResponse_results_dataType(str, Enum):
    Boolean = "boolean",
    True_ = "true",
    False_ = "false",
    Text = "text",
    Numeric = "numeric",
    Date = "date",
    TwoZeroTwoOne = "2021",
    Select = "select",
    EnumValues = "enumValues",
    Multi_select = "multi_select",




from enum import Enum

class SmartregisterGetResponse_specCategory(str, Enum):
    TestReports = "Test Reports",
    ShopDrawings = "Shop Drawings",
    Schedules = "Schedules",
    Samples = "Samples",
    SampleWarranty = "Sample Warranty",
    Reports = "Reports",
    QualificationData = "Qualification Data",
    QUALITYASSURANCE = "QUALITY ASSURANCE",
    ProductData = "Product Data",
    PerformanceData = "Performance Data",
    MfgInstructions = "Mfg. Instructions",
    MeetingConferences = "Meeting/Conferences",
    Drawings = "Drawings",
    Delegated = "Delegated",
    Certifications = "Certifications",
    Certificates = "Certificates",
    Calculations = "Calculations",
    AtticStock = "Attic Stock",
    Demonstrations = "Demonstrations",
    GeneralWarranties = "General Warranties",
    OMManuals = "O&M Manuals",
    SpecialWarranties = "Special Warranties",
    LEED = "LEED",
    As_ = "As",
    TESTSANDINSPECTIONS = "TESTS AND INSPECTIONS",
    General = "General",
    ManufacturersInstructions = "Manufacturers Instructions",
    Substitutions = "Substitutions",
    MixDesign = "Mix Design",
    Others = "Others",




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .properties_get_response_type import PropertiesGetResponse_type

@dataclass
class PropertiesGetResponse(Parsable):
    # A true/false flag to indicate whether this is a pre-defined attribute or not.
    built_in: Optional[bool] = None
    # Not relevant
    category: Optional[str] = None
    # Custom attribute belongs to the component template
    component_template_id: Optional[str] = None
    # Default value of the attribute definition.
    default_value: Optional[str] = None
    # A true/false flag to indicate whether this attribute should be shown in the BIM 360 Cost Management or generated documents by default.
    default_visibility: Optional[bool] = None
    # Unique identifier (UUID) of the attribute definition.
    id: Optional[UUID] = None
    # Name of a attribute definition.Max length: 1024
    name: Optional[str] = None
    # List of items for an ``options`` type attribute.
    options: Optional[list[str]] = None
    # The position of the attribute definition as displayed in BIM 360 Cost Management.
    position: Optional[float] = None
    # Type of the attribute definition.Possible values: ``text``, ``multiline``, ``richtext``, ``options``, ``boolean``, ``integer``, ``percent``, ``currency``, ``number``, ``datetime``
    type: Optional[PropertiesGetResponse_type] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> PropertiesGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: PropertiesGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return PropertiesGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .properties_get_response_type import PropertiesGetResponse_type

        from .properties_get_response_type import PropertiesGetResponse_type

        fields: dict[str, Callable[[Any], None]] = {
            "builtIn": lambda n : setattr(self, 'built_in', n.get_bool_value()),
            "category": lambda n : setattr(self, 'category', n.get_str_value()),
            "componentTemplateId": lambda n : setattr(self, 'component_template_id', n.get_str_value()),
            "defaultValue": lambda n : setattr(self, 'default_value', n.get_str_value()),
            "defaultVisibility": lambda n : setattr(self, 'default_visibility', n.get_bool_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "options": lambda n : setattr(self, 'options', n.get_collection_of_primitive_values(str)),
            "position": lambda n : setattr(self, 'position', n.get_float_value()),
            "type": lambda n : setattr(self, 'type', n.get_enum_value(PropertiesGetResponse_type)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_bool_value("builtIn", self.built_in)
        writer.write_str_value("category", self.category)
        writer.write_str_value("componentTemplateId", self.component_template_id)
        writer.write_str_value("defaultValue", self.default_value)
        writer.write_bool_value("defaultVisibility", self.default_visibility)
        writer.write_uuid_value("id", self.id)
        writer.write_str_value("name", self.name)
        writer.write_collection_of_primitive_values("options", self.options)
        writer.write_float_value("position", self.position)
        writer.write_enum_value("type", self.type)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .with_task_get_response_assigned_to_type import WithTaskGetResponse_assignedToType
    from .with_task_get_response_permitted_actions import WithTaskGetResponse_permittedActions
    from .with_task_get_response_status import WithTaskGetResponse_status

@dataclass
class WithTaskGetResponse(Parsable):
    # The Autodesk ID or member group ID of the ``user``, ``company``, or ``role`` assigned to the task.
    assigned_to: Optional[str] = None
    # Specifies whether the task is assigned to a user, company, or role.Possible values: ``1`` (user), ``2`` (company), ``3`` (role).
    assigned_to_type: Optional[WithTaskGetResponse_assignedToType] = None
    # The date and time when the task was completed, formatted as YYYY-MM-DDTHH:mm:ss.SSSSSSZ (ISO 8601) in UTC. For example, ``2025-01-20T12:00:00.198466Z``.
    completed_at: Optional[datetime.datetime] = None
    # The Autodesk ID of the user who completed the task.
    completed_by: Optional[str] = None
    # The date and time when the task was originally created, formatted as YYYY-MM-DDTHH:mm:ss.SSSSSSZ (ISO 8601) in UTC. For example, ``2025-01-20T12:00:00.198466Z``.
    created_at: Optional[datetime.datetime] = None
    # The Autodesk ID of the user who created the task.
    created_by: Optional[str] = None
    # The internal, globally unique identifier (UUID) for the task.
    id: Optional[UUID] = None
    # ``true``: the task is required to complete the step.``false``: (default) the task is not required to complete the step.
    is_required: Optional[bool] = None
    # A list of actions that the user is allowed to perform on the task within the submittal workflow.
    permitted_actions: Optional[list[WithTaskGetResponse_permittedActions]] = None
    # The date and time when the response was added, formatted as YYYY-MM-DDTHH:mm:ss.SSSSSSZ (ISO 8601) in UTC. For example, ``2025-01-20T12:00:00.198466Z``.
    responded_at: Optional[datetime.datetime] = None
    # The Autodesk ID of the user who provided the response to the task.
    responded_by: Optional[str] = None
    # The content of the response comment, providing feedback or instructions related to the task.
    response_comment: Optional[str] = None
    # The ID of the response associated with the task, linking to the specific feedback or action taken.
    response_id: Optional[UUID] = None
    # The date and time when the related step was marked as started (``In Progress``), formatted as YYYY-MM-DDTHH:mm:ss.SSSSSSZ (ISO 8601) in UTC. For example, ``2025-01-20T12:00:00.198466Z``.
    started_at: Optional[datetime.datetime] = None
    # The current status of the task. Possible values: ``not-started``, ``in-progress``, ``completed``.
    status: Optional[WithTaskGetResponse_status] = None
    # The due date of the related step, formatted as YYYY-MM-DD (ISO 8601) in UTC. For example, ``2025-01-20``.
    step_due_date: Optional[str] = None
    # The ID of the review step associated with the task.
    step_id: Optional[UUID] = None
    # The date and time when the task was last updated, formatted as YYYY-MM-DDTHH:mm:ss.SSSSSSZ (ISO 8601) in UTC. For example, ``2025-01-20T12:00:00.198466Z``.
    updated_at: Optional[datetime.datetime] = None
    # The Autodesk ID of the user who last updated the task.
    updated_by: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithTaskGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithTaskGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithTaskGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .with_task_get_response_assigned_to_type import WithTaskGetResponse_assignedToType
        from .with_task_get_response_permitted_actions import WithTaskGetResponse_permittedActions
        from .with_task_get_response_status import WithTaskGetResponse_status

        from .with_task_get_response_assigned_to_type import WithTaskGetResponse_assignedToType
        from .with_task_get_response_permitted_actions import WithTaskGetResponse_permittedActions
        from .with_task_get_response_status import WithTaskGetResponse_status

        fields: dict[str, Callable[[Any], None]] = {
            "assignedTo": lambda n : setattr(self, 'assigned_to', n.get_str_value()),
            "assignedToType": lambda n : setattr(self, 'assigned_to_type', n.get_enum_value(WithTaskGetResponse_assignedToType)),
            "completedAt": lambda n : setattr(self, 'completed_at', n.get_datetime_value()),
            "completedBy": lambda n : setattr(self, 'completed_by', n.get_str_value()),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "createdBy": lambda n : setattr(self, 'created_by', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "isRequired": lambda n : setattr(self, 'is_required', n.get_bool_value()),
            "permittedActions": lambda n : setattr(self, 'permitted_actions', n.get_collection_of_object_values(WithTaskGetResponse_permittedActions)),
            "respondedAt": lambda n : setattr(self, 'responded_at', n.get_datetime_value()),
            "respondedBy": lambda n : setattr(self, 'responded_by', n.get_str_value()),
            "responseComment": lambda n : setattr(self, 'response_comment', n.get_str_value()),
            "responseId": lambda n : setattr(self, 'response_id', n.get_uuid_value()),
            "startedAt": lambda n : setattr(self, 'started_at', n.get_datetime_value()),
            "status": lambda n : setattr(self, 'status', n.get_enum_value(WithTaskGetResponse_status)),
            "stepDueDate": lambda n : setattr(self, 'step_due_date', n.get_str_value()),
            "stepId": lambda n : setattr(self, 'step_id', n.get_uuid_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
            "updatedBy": lambda n : setattr(self, 'updated_by', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("assignedTo", self.assigned_to)
        writer.write_enum_value("assignedToType", self.assigned_to_type)
        writer.write_datetime_value("completedAt", self.completed_at)
        writer.write_str_value("completedBy", self.completed_by)
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_str_value("createdBy", self.created_by)
        writer.write_uuid_value("id", self.id)
        writer.write_bool_value("isRequired", self.is_required)
        writer.write_collection_of_object_values("permittedActions", self.permitted_actions)
        writer.write_datetime_value("respondedAt", self.responded_at)
        writer.write_str_value("respondedBy", self.responded_by)
        writer.write_str_value("responseComment", self.response_comment)
        writer.write_uuid_value("responseId", self.response_id)
        writer.write_datetime_value("startedAt", self.started_at)
        writer.write_enum_value("status", self.status)
        writer.write_str_value("stepDueDate", self.step_due_date)
        writer.write_uuid_value("stepId", self.step_id)
        writer.write_datetime_value("updatedAt", self.updated_at)
        writer.write_str_value("updatedBy", self.updated_by)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .forms_post_response_weather_hourly_weather import FormsPostResponse_weather_hourlyWeather
    from .forms_post_response_weather_provider import FormsPostResponse_weather_provider

@dataclass
class FormsPostResponse_weather(Parsable):
    # Weather information for specific hours (07:00:00, 12:00:00, 16:00:00).
    hourly_weather: Optional[list[FormsPostResponse_weather_hourlyWeather]] = None
    # A percentage value indicating the humidity over the course of the day.
    humidity: Optional[float] = None
    # Amount of precipitation accumulated throughout the day.
    precipitation_accumulation: Optional[float] = None
    # Indicates the measurement unit of the ``precipitationAccumulation``.
    precipitation_accumulation_unit: Optional[str] = None
    # Indicates the source of the weather data.Possible values: ``darksky``, ``weatherkit``
    provider: Optional[FormsPostResponse_weather_provider] = None
    # Indicates the measurement unit of the ``windSpeed`` and ``windGust``.
    speed_unit: Optional[str] = None
    # A code describing the the weather (e.g. Clear, PartlyCloudy).
    summary_key: Optional[str] = None
    # Maximum temperature during the day.
    temperature_max: Optional[float] = None
    # Minimum temperature during the day.
    temperature_min: Optional[float] = None
    # Indicates the measurement unit of the temperature values e.g. ``temperatureMin``, ``temperatureMax``, ``temp``.
    temperature_unit: Optional[str] = None
    # Direction of the wind, in degrees.
    wind_bearing: Optional[float] = None
    # Maximum wind speed observed throughout the day.
    wind_gust: Optional[float] = None
    # Average wind speed observed throughout the day.
    wind_speed: Optional[float] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> FormsPostResponse_weather:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: FormsPostResponse_weather
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return FormsPostResponse_weather()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .forms_post_response_weather_hourly_weather import FormsPostResponse_weather_hourlyWeather
        from .forms_post_response_weather_provider import FormsPostResponse_weather_provider

        from .forms_post_response_weather_hourly_weather import FormsPostResponse_weather_hourlyWeather
        from .forms_post_response_weather_provider import FormsPostResponse_weather_provider

        fields: dict[str, Callable[[Any], None]] = {
            "hourlyWeather": lambda n : setattr(self, 'hourly_weather', n.get_collection_of_object_values(FormsPostResponse_weather_hourlyWeather)),
            "humidity": lambda n : setattr(self, 'humidity', n.get_float_value()),
            "precipitationAccumulation": lambda n : setattr(self, 'precipitation_accumulation', n.get_float_value()),
            "precipitationAccumulationUnit": lambda n : setattr(self, 'precipitation_accumulation_unit', n.get_str_value()),
            "provider": lambda n : setattr(self, 'provider', n.get_enum_value(FormsPostResponse_weather_provider)),
            "speedUnit": lambda n : setattr(self, 'speed_unit', n.get_str_value()),
            "summaryKey": lambda n : setattr(self, 'summary_key', n.get_str_value()),
            "temperatureMax": lambda n : setattr(self, 'temperature_max', n.get_float_value()),
            "temperatureMin": lambda n : setattr(self, 'temperature_min', n.get_float_value()),
            "temperatureUnit": lambda n : setattr(self, 'temperature_unit', n.get_str_value()),
            "windBearing": lambda n : setattr(self, 'wind_bearing', n.get_float_value()),
            "windGust": lambda n : setattr(self, 'wind_gust', n.get_float_value()),
            "windSpeed": lambda n : setattr(self, 'wind_speed', n.get_float_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_object_values("hourlyWeather", self.hourly_weather)
        writer.write_float_value("humidity", self.humidity)
        writer.write_float_value("precipitationAccumulation", self.precipitation_accumulation)
        writer.write_str_value("precipitationAccumulationUnit", self.precipitation_accumulation_unit)
        writer.write_enum_value("provider", self.provider)
        writer.write_str_value("speedUnit", self.speed_unit)
        writer.write_str_value("summaryKey", self.summary_key)
        writer.write_float_value("temperatureMax", self.temperature_max)
        writer.write_float_value("temperatureMin", self.temperature_min)
        writer.write_str_value("temperatureUnit", self.temperature_unit)
        writer.write_float_value("windBearing", self.wind_bearing)
        writer.write_float_value("windGust", self.wind_gust)
        writer.write_float_value("windSpeed", self.wind_speed)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .photos_filter_post_response_results_signed_urls import PhotosFilterPostResponse_results_signedUrls
    from .photos_filter_post_response_results_urls import PhotosFilterPostResponse_results_urls

@dataclass
class PhotosFilterPostResponse_results(Parsable):
    # The createdAt property
    created_at: Optional[datetime.datetime] = None
    # The createdBy property
    created_by: Optional[str] = None
    # The deletedAt property
    deleted_at: Optional[datetime.datetime] = None
    # The deletedBy property
    deleted_by: Optional[str] = None
    # The description property
    description: Optional[str] = None
    # The id property
    id: Optional[UUID] = None
    # The isPublic property
    is_public: Optional[bool] = None
    # The latitude property
    latitude: Optional[float] = None
    # The locked property
    locked: Optional[bool] = None
    # The longitude property
    longitude: Optional[float] = None
    # The mediaType property
    media_type: Optional[str] = None
    # The projectId property
    project_id: Optional[UUID] = None
    # The signedUrls property
    signed_urls: Optional[PhotosFilterPostResponse_results_signedUrls] = None
    # The size property
    size: Optional[float] = None
    # The takenAt property
    taken_at: Optional[datetime.datetime] = None
    # The title property
    title: Optional[str] = None
    # The type property
    type: Optional[str] = None
    # The updatedAt property
    updated_at: Optional[datetime.datetime] = None
    # The updatedBy property
    updated_by: Optional[str] = None
    # The urls property
    urls: Optional[PhotosFilterPostResponse_results_urls] = None
    # The userCreatedAt property
    user_created_at: Optional[datetime.datetime] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> PhotosFilterPostResponse_results:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: PhotosFilterPostResponse_results
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return PhotosFilterPostResponse_results()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .photos_filter_post_response_results_signed_urls import PhotosFilterPostResponse_results_signedUrls
        from .photos_filter_post_response_results_urls import PhotosFilterPostResponse_results_urls

        from .photos_filter_post_response_results_signed_urls import PhotosFilterPostResponse_results_signedUrls
        from .photos_filter_post_response_results_urls import PhotosFilterPostResponse_results_urls

        fields: dict[str, Callable[[Any], None]] = {
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "createdBy": lambda n : setattr(self, 'created_by', n.get_str_value()),
            "deletedAt": lambda n : setattr(self, 'deleted_at', n.get_datetime_value()),
            "deletedBy": lambda n : setattr(self, 'deleted_by', n.get_str_value()),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "isPublic": lambda n : setattr(self, 'is_public', n.get_bool_value()),
            "latitude": lambda n : setattr(self, 'latitude', n.get_float_value()),
            "locked": lambda n : setattr(self, 'locked', n.get_bool_value()),
            "longitude": lambda n : setattr(self, 'longitude', n.get_float_value()),
            "mediaType": lambda n : setattr(self, 'media_type', n.get_str_value()),
            "projectId": lambda n : setattr(self, 'project_id', n.get_uuid_value()),
            "signedUrls": lambda n : setattr(self, 'signed_urls', n.get_object_value(PhotosFilterPostResponse_results_signedUrls)),
            "size": lambda n : setattr(self, 'size', n.get_float_value()),
            "takenAt": lambda n : setattr(self, 'taken_at', n.get_datetime_value()),
            "title": lambda n : setattr(self, 'title', n.get_str_value()),
            "type": lambda n : setattr(self, 'type', n.get_str_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
            "updatedBy": lambda n : setattr(self, 'updated_by', n.get_str_value()),
            "urls": lambda n : setattr(self, 'urls', n.get_object_value(PhotosFilterPostResponse_results_urls)),
            "userCreatedAt": lambda n : setattr(self, 'user_created_at', n.get_datetime_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_str_value("createdBy", self.created_by)
        writer.write_datetime_value("deletedAt", self.deleted_at)
        writer.write_str_value("deletedBy", self.deleted_by)
        writer.write_str_value("description", self.description)
        writer.write_uuid_value("id", self.id)
        writer.write_bool_value("isPublic", self.is_public)
        writer.write_float_value("latitude", self.latitude)
        writer.write_bool_value("locked", self.locked)
        writer.write_float_value("longitude", self.longitude)
        writer.write_str_value("mediaType", self.media_type)
        writer.write_uuid_value("projectId", self.project_id)
        writer.write_object_value("signedUrls", self.signed_urls)
        writer.write_float_value("size", self.size)
        writer.write_datetime_value("takenAt", self.taken_at)
        writer.write_str_value("title", self.title)
        writer.write_str_value("type", self.type)
        writer.write_datetime_value("updatedAt", self.updated_at)
        writer.write_str_value("updatedBy", self.updated_by)
        writer.write_object_value("urls", self.urls)
        writer.write_datetime_value("userCreatedAt", self.user_created_at)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .documents_get_response_results_approve_status_value import DocumentsGetResponse_results_approveStatus_value

@dataclass
class DocumentsGetResponse_results_approveStatus(Parsable):
    """
    The approval status of the file version.For more information, see `File Review Statuses <https://help.autodesk.com/view/DOCS/ENU/?guid=BIM360D_Document_Management_About_Reviews_Reviews_FAQs_Reference_html#file-review-statuses>`_ documentation.
    """
    # The customized label of the approval status.Max length: 255
    label: Optional[str] = None
    # The value of the approval status.Possible values: ``APPROVED``, ``REJECTED``
    value: Optional[DocumentsGetResponse_results_approveStatus_value] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> DocumentsGetResponse_results_approveStatus:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: DocumentsGetResponse_results_approveStatus
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return DocumentsGetResponse_results_approveStatus()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .documents_get_response_results_approve_status_value import DocumentsGetResponse_results_approveStatus_value

        from .documents_get_response_results_approve_status_value import DocumentsGetResponse_results_approveStatus_value

        fields: dict[str, Callable[[Any], None]] = {
            "label": lambda n : setattr(self, 'label', n.get_str_value()),
            "value": lambda n : setattr(self, 'value', n.get_enum_value(DocumentsGetResponse_results_approveStatus_value)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("label", self.label)
        writer.write_enum_value("value", self.value)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .workflows_post_response_additional_options import WorkflowsPostResponse_additionalOptions
    from .workflows_post_response_approval_status_options import WorkflowsPostResponse_approvalStatusOptions
    from .workflows_post_response_attached_attributes import WorkflowsPostResponse_attachedAttributes
    from .workflows_post_response_copy_files_options import WorkflowsPostResponse_copyFilesOptions
    from .workflows_post_response_status import WorkflowsPostResponse_status
    from .workflows_post_response_steps import WorkflowsPostResponse_steps
    from .workflows_post_response_update_attributes_options import WorkflowsPostResponse_updateAttributesOptions

@dataclass
class WorkflowsPostResponse(Parsable):
    # Workflow-level settings that control whether the initiator can modify certain fields when starting a review.
    additional_options: Optional[WorkflowsPostResponse_additionalOptions] = None
    # A list of approval status options defined for this workflow.It includes two built-in options by default (typically ``APPROVED`` and ``REJECTED``), and may also include custom statuses added by the user.
    approval_status_options: Optional[list[WorkflowsPostResponse_approvalStatusOptions]] = None
    # (``Update Attributes`` in the UI)The list of attributes added in the ``Update Attributes`` action.These attributes will be applied to the approved files in the target folder, or optionally also in the source folder depending on the configuration.
    attached_attributes: Optional[list[WorkflowsPostResponse_attachedAttributes]] = None
    # (``Copy approved files`` in the UI) The configuration for copying approved files to a target folder when the review is complete.
    copy_files_options: Optional[WorkflowsPostResponse_copyFilesOptions] = None
    # The date and time when the workflow was created.
    created_at: Optional[datetime.datetime] = None
    # A description of the workflow.Max length: 4096
    description: Optional[str] = None
    # The unique identifier of the approval workflow returned in the response.This ID can be used in subsequent API calls to reference this workflow.
    id: Optional[UUID] = None
    # The name of the workflow. It must be unique within the project.Max length: 255
    name: Optional[str] = None
    # A custom note associated with the workflow. Visible to all reviewers during the review process.Max length: 4096
    notes: Optional[str] = None
    # The current status of the approval workflow. Possible values:``ACTIVE``: the workflow is active and available for use.``INACTIVE``: the workflow has been deactivated and cannot be used to create new reviews.
    status: Optional[WorkflowsPostResponse_status] = None
    # A list of steps defined in the approval workflow. Each step defines who reviews the files, how long they have, and whether it involves multiple reviewers.
    steps: Optional[list[WorkflowsPostResponse_steps]] = None
    # The configuration for applying attribute updates when a review is completed.This applies only if the workflow includes a file copy action and the ``Update Attributes`` action is enabled.
    update_attributes_options: Optional[WorkflowsPostResponse_updateAttributesOptions] = None
    # The date and time when the workflow was last updated.
    updated_at: Optional[datetime.datetime] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WorkflowsPostResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WorkflowsPostResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WorkflowsPostResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .workflows_post_response_additional_options import WorkflowsPostResponse_additionalOptions
        from .workflows_post_response_approval_status_options import WorkflowsPostResponse_approvalStatusOptions
        from .workflows_post_response_attached_attributes import WorkflowsPostResponse_attachedAttributes
        from .workflows_post_response_copy_files_options import WorkflowsPostResponse_copyFilesOptions
        from .workflows_post_response_status import WorkflowsPostResponse_status
        from .workflows_post_response_steps import WorkflowsPostResponse_steps
        from .workflows_post_response_update_attributes_options import WorkflowsPostResponse_updateAttributesOptions

        from .workflows_post_response_additional_options import WorkflowsPostResponse_additionalOptions
        from .workflows_post_response_approval_status_options import WorkflowsPostResponse_approvalStatusOptions
        from .workflows_post_response_attached_attributes import WorkflowsPostResponse_attachedAttributes
        from .workflows_post_response_copy_files_options import WorkflowsPostResponse_copyFilesOptions
        from .workflows_post_response_status import WorkflowsPostResponse_status
        from .workflows_post_response_steps import WorkflowsPostResponse_steps
        from .workflows_post_response_update_attributes_options import WorkflowsPostResponse_updateAttributesOptions

        fields: dict[str, Callable[[Any], None]] = {
            "additionalOptions": lambda n : setattr(self, 'additional_options', n.get_object_value(WorkflowsPostResponse_additionalOptions)),
            "approvalStatusOptions": lambda n : setattr(self, 'approval_status_options', n.get_collection_of_object_values(WorkflowsPostResponse_approvalStatusOptions)),
            "attachedAttributes": lambda n : setattr(self, 'attached_attributes', n.get_collection_of_object_values(WorkflowsPostResponse_attachedAttributes)),
            "copyFilesOptions": lambda n : setattr(self, 'copy_files_options', n.get_object_value(WorkflowsPostResponse_copyFilesOptions)),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "notes": lambda n : setattr(self, 'notes', n.get_str_value()),
            "status": lambda n : setattr(self, 'status', n.get_enum_value(WorkflowsPostResponse_status)),
            "steps": lambda n : setattr(self, 'steps', n.get_collection_of_object_values(WorkflowsPostResponse_steps)),
            "updateAttributesOptions": lambda n : setattr(self, 'update_attributes_options', n.get_object_value(WorkflowsPostResponse_updateAttributesOptions)),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_object_value("additionalOptions", self.additional_options)
        writer.write_collection_of_object_values("approvalStatusOptions", self.approval_status_options)
        writer.write_collection_of_object_values("attachedAttributes", self.attached_attributes)
        writer.write_object_value("copyFilesOptions", self.copy_files_options)
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_str_value("description", self.description)
        writer.write_uuid_value("id", self.id)
        writer.write_str_value("name", self.name)
        writer.write_str_value("notes", self.notes)
        writer.write_enum_value("status", self.status)
        writer.write_collection_of_object_values("steps", self.steps)
        writer.write_object_value("updateAttributesOptions", self.update_attributes_options)
        writer.write_datetime_value("updatedAt", self.updated_at)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .exports_post_request_body_options import ExportsPostRequestBody_options

@dataclass
class ExportsPostRequestBody(Parsable):
    # A list of file version URNs. A maximum of 200 files may be included.
    file_versions: Optional[list[str]] = None
    # The criteria for the markups and links to include with the exported files.Note that unpublished markups are those visible only to their creator.
    options: Optional[ExportsPostRequestBody_options] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ExportsPostRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ExportsPostRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ExportsPostRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .exports_post_request_body_options import ExportsPostRequestBody_options

        from .exports_post_request_body_options import ExportsPostRequestBody_options

        fields: dict[str, Callable[[Any], None]] = {
            "fileVersions": lambda n : setattr(self, 'file_versions', n.get_collection_of_primitive_values(str)),
            "options": lambda n : setattr(self, 'options', n.get_object_value(ExportsPostRequestBody_options)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_primitive_values("fileVersions", self.file_versions)
        writer.write_object_value("options", self.options)
    




from enum import Enum

class ValuesBatchUpdatePutRequestBody_tabularValues_table(str, Enum):
    WorklogEntries = "worklogEntries",
    MaterialsEntries = "materialsEntries",
    EquipmentEntries = "equipmentEntries",




from enum import Enum

class GetSortQueryParameterType(str, Enum):
    Name = "name",
    NameAsc = "name asc",
    Desc = "desc",
    SortFieldAscOrDesc = "sort=<field> [asc or desc]",
    Title = "title",
    Version = "version",
    LastUpdatedAt = "lastUpdatedAt",
    UpdatedByName = "updatedByName",
    SortNameAsc = "sort=name asc",
    SortLastUpdatedAtDesc = "sort=lastUpdatedAt desc",




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .values_batch_update_put_request_body_tabular_values_columns import ValuesBatchUpdatePutRequestBody_tabularValues_columns
    from .values_batch_update_put_request_body_tabular_values_table import ValuesBatchUpdatePutRequestBody_tabularValues_table

@dataclass
class ValuesBatchUpdatePutRequestBody_tabularValues(Parsable):
    # The set of values for the columns in the row.
    columns: Optional[list[ValuesBatchUpdatePutRequestBody_tabularValues_columns]] = None
    # The ID of the table row. You need to generarte the ID (UUID) for the row.
    id: Optional[UUID] = None
    # The table to update.Possible values: ``worklogEntries``, ``materialsEntries``, ``equipmentEntries``
    table: Optional[ValuesBatchUpdatePutRequestBody_tabularValues_table] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ValuesBatchUpdatePutRequestBody_tabularValues:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ValuesBatchUpdatePutRequestBody_tabularValues
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ValuesBatchUpdatePutRequestBody_tabularValues()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .values_batch_update_put_request_body_tabular_values_columns import ValuesBatchUpdatePutRequestBody_tabularValues_columns
        from .values_batch_update_put_request_body_tabular_values_table import ValuesBatchUpdatePutRequestBody_tabularValues_table

        from .values_batch_update_put_request_body_tabular_values_columns import ValuesBatchUpdatePutRequestBody_tabularValues_columns
        from .values_batch_update_put_request_body_tabular_values_table import ValuesBatchUpdatePutRequestBody_tabularValues_table

        fields: dict[str, Callable[[Any], None]] = {
            "columns": lambda n : setattr(self, 'columns', n.get_collection_of_object_values(ValuesBatchUpdatePutRequestBody_tabularValues_columns)),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "table": lambda n : setattr(self, 'table', n.get_enum_value(ValuesBatchUpdatePutRequestBody_tabularValues_table)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_object_values("columns", self.columns)
        writer.write_uuid_value("id", self.id)
        writer.write_enum_value("table", self.table)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .templates_get_response_results_steps import TemplatesGetResponse_results_steps
    from .templates_get_response_results_watchers import TemplatesGetResponse_results_watchers

@dataclass
class TemplatesGetResponse_results(Parsable):
    # The date and time when the template was originally created, formatted as YYYY-MM-DDTHH:mm:ss.SSSSSSZ (ISO 8601) in UTC. For example, ``2025-01-20T12:00:00.198466Z``.
    created_at: Optional[datetime.datetime] = None
    # The Autodesk ID of the user that created the template.
    created_by: Optional[str] = None
    # The internal, globally unique identifier (UUID) for the template. Each template in the workflow has a unique ID.
    id: Optional[UUID] = None
    # The name of the template.
    name: Optional[str] = None
    # The list of steps included in the template. Each step defines specific actions and associated tasks.
    steps: Optional[list[TemplatesGetResponse_results_steps]] = None
    # The date and time when the template was last updated, formatted as YYYY-MM-DDTHH:mm:ss.SSSSSSZ (ISO 8601) in UTC. For example, ``2025-01-20T12:00:00.198466Z``.
    updated_at: Optional[datetime.datetime] = None
    # The Autodesk ID of the user that last updated the template.
    updated_by: Optional[str] = None
    # A list of project watchers, who can be individual users, roles, or companies.
    watchers: Optional[list[TemplatesGetResponse_results_watchers]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> TemplatesGetResponse_results:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: TemplatesGetResponse_results
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return TemplatesGetResponse_results()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .templates_get_response_results_steps import TemplatesGetResponse_results_steps
        from .templates_get_response_results_watchers import TemplatesGetResponse_results_watchers

        from .templates_get_response_results_steps import TemplatesGetResponse_results_steps
        from .templates_get_response_results_watchers import TemplatesGetResponse_results_watchers

        fields: dict[str, Callable[[Any], None]] = {
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "createdBy": lambda n : setattr(self, 'created_by', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "steps": lambda n : setattr(self, 'steps', n.get_collection_of_object_values(TemplatesGetResponse_results_steps)),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
            "updatedBy": lambda n : setattr(self, 'updated_by', n.get_str_value()),
            "watchers": lambda n : setattr(self, 'watchers', n.get_collection_of_object_values(TemplatesGetResponse_results_watchers)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_str_value("createdBy", self.created_by)
        writer.write_uuid_value("id", self.id)
        writer.write_str_value("name", self.name)
        writer.write_collection_of_object_values("steps", self.steps)
        writer.write_datetime_value("updatedAt", self.updated_at)
        writer.write_str_value("updatedBy", self.updated_by)
        writer.write_collection_of_object_values("watchers", self.watchers)
    

